/* License of this file: CC0 Public Domain Dedication along
   see http://creativecommons.org/publicdomain/zero/1.0/ */

#include <stdio.h>
#include <stdlib.h>

#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>

static char *font[] = {"../../../font/Vera.ttf,size=18"};
static mbtk_backend_t be;
static mbtk_display_t disp;

/* Initialize be and disp and set up f3d for default style. Exits on error. */
static void example_init_sdl2(int *argc, char **argv[])
{
	if (mbtk_sdl2_backend(&be) != 0) {
		fprintf(stderr, "Unable to set up the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_be_init(&be) != 0) {
		fprintf(stderr, "Unable to initialize the sdl2 backend\n");
		exit(1);
	}

	if (mbtk_display_init(&be, &disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with the sdl2 backend\n");
		exit(1);
	}

	/* Set default style and font ID for the display */
	mbtk_f3d_style(&disp);
	disp.default_fid = mbtk_resolve_font(&disp, 1, font);
}

static mbtk_event_handled_t button_clk(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_pane_t *pane = user_data;
	mbtk_pane_set_percent(pane, 0.7);
/*	mbtk_pane_set_pixel(pane, 300);*/
	return 0;
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_box_t *root;
	mbtk_label_t *lab;
	mbtk_button_t *btn;
	mbtk_pane_t *pane;

	example_init_sdl2(&argc, &argv);

	/* create a large, normal (non-modal, non-menu) window. -1 for coord means
	   we do not have a request on placement coords; the last two are recommended
	   initial window size */
	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk pane", -1, -1, 800, 200);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	/* create the root widget as a vbox so child widgets are packed vertically;
	   default alignment is "justify", which means excess space are put between
	   the widgets. */
	root = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &root->w);

	/* Create the horizontal pane */
	pane = mbtk_pane_new(NULL, 1);
	mbtk_pane_set_span(pane, HVBOX_FILL);
	mbtk_box_add_widget(root, &pane->w, 0);

	/* Create and add a label on the left */
	lab = mbtk_label_new(NULL, "Hello\nworld!");
	mbtk_pane_set_child(pane, 0, &lab->w);

	/* Create an add a button on the right */
	btn = mbtk_button_new_with_label("Click me!");
	mbtk_button_callback_click(btn, button_clk, pane);
	mbtk_pane_set_child(pane, 1, &btn->w);


	/* main loop: run the GUI until win is closed */
	mbtk_run_window(win);

	/* Uninit everyhting at once */
	mbtk_uninit();
}
