/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static mbtk_mocu_t *mbtk_glfw_mocu_create_named(mbtk_display_t *disp, mbtk_mocu_named_t name)
{
	int shape;

	switch(name) {
		case MBTK_MOCU_ARROW:     shape = GLFW_ARROW_CURSOR; break;
		case MBTK_MOCU_IBEAM:     shape = GLFW_IBEAM_CURSOR; break;
		case MBTK_MOCU_CROSSHAIR: shape = GLFW_CROSSHAIR_CURSOR; break;
		case MBTK_MOCU_HAND:      shape = GLFW_HAND_CURSOR; break;
		case MBTK_MOCU_HARROW:    shape = GLFW_HRESIZE_CURSOR; break;
		case MBTK_MOCU_VARROW:    shape = GLFW_VRESIZE_CURSOR; break;
		default: return NULL;
	}


	return (mbtk_mocu_t *)glfwCreateStandardCursor(shape);
}

static mbtk_mocu_t *mbtk_glfw_mocu_create_pixmap(mbtk_display_t *disp, int width, int height, unsigned const char *px, unsigned const char *mask, int hot_x, int hot_y)
{
	GLFWimage img; /* RGBA */
	int n, end, bc;
	unsigned char *o;
	unsigned int currp, currm;

	end = width*height;

	img.width = width;
	img.height = height;
	img.pixels = malloc(end*4);

	o = img.pixels;
	currp = *px;
	currm = *mask;

	for(bc = n = 0; n < end; n++) {
		int p = currp & 1, m = currm & 1;
		currp = currp >> 1;
		currm = currm >> 1;

		if (m == 0) { /* transparent */
			o[0] = o[1] = o[2] = o[3] = 0;
		}
		else {
			o[0] = o[1] = o[2] = p ? 255 : 0;
			o[3] = 255;
		}
		o+=4;

		bc++;
		if (bc > 7) {
			bc = 0;
			px++;
			mask++;
			currp = *px;
			currm = *mask;
		}
	}

	return (mbtk_mocu_t *)glfwCreateCursor(&img, hot_x, hot_y);
}

static void mbtk_glfw_mocu_free_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs)
{
	glfwDestroyCursor((GLFWcursor *)curs);
}

static void mbtk_glfw_mocu_set_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs)
{
	glfw_win_t *xw = (glfw_win_t *)disp->topwin;
	glfwSetCursor(xw->window, (GLFWcursor *)curs);
}


