/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static char mbtk_glfw_button_xlate(int button)
{
	switch(button) {
		case GLFW_MOUSE_BUTTON_LEFT:   return MBTK_MB_LEFT;
		case GLFW_MOUSE_BUTTON_MIDDLE: return MBTK_MB_MIDDLE;
		case GLFW_MOUSE_BUTTON_RIGHT:  return MBTK_MB_RIGHT;
		case GLFW_MOUSE_BUTTON_4:      return MBTK_MB_EXTRA1;
		case GLFW_MOUSE_BUTTON_5:      return MBTK_MB_EXTRA2;
		case GLFW_MOUSE_BUTTON_6:      return MBTK_MB_EXTRA3;
		case GLFW_MOUSE_BUTTON_7:      return MBTK_MB_EXTRA4;
		case GLFW_MOUSE_BUTTON_8:      return MBTK_MB_EXTRA5;
	}
	return MBTK_MB_UNKNOWN;
}

#include "../common/utf8.h"

static mbtk_keymod_t mbtk_glfw_key_mods(int mod)
{
	mbtk_keymod_t res = 0;
	if (mod & GLFW_MOD_SHIFT)   res |= MBTK_KM_SHIFT;
	if (mod & GLFW_MOD_ALT)     res |= MBTK_KM_ALT;
	if (mod & GLFW_MOD_CONTROL) res |= MBTK_KM_CTRL;
	return res;
}

static mbtk_keysym_t mbtk_glfw_key_sym(int key, int mods, mbtk_keymod_t *mod)
{
	int numlock;

#ifdef GLFW_MOD_NUM_LOCK
	numlock = !!(mods & GLFW_MOD_NUM_LOCK);
#else
	numlock = 0;
#endif

	if ((key >= 'A') && (key <= 'Z'))
		return key - 'A' + 'a';
	if ((key >= 32) && (key <= 127))
		return key;

	switch(key) {
		case GLFW_KEY_BACKSPACE:        return MBTK_KS_BACKSPACE;
		case GLFW_KEY_TAB:              return MBTK_KS_TAB;
		case GLFW_KEY_KP_ENTER:         (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_ENTER;
		case GLFW_KEY_ENTER:            return MBTK_KS_ENTER;
		case GLFW_KEY_ESCAPE:           return MBTK_KS_ESC;

		case GLFW_KEY_F1:               return MBTK_KS_FN+1;
		case GLFW_KEY_F2:               return MBTK_KS_FN+2;
		case GLFW_KEY_F3:               return MBTK_KS_FN+3;
		case GLFW_KEY_F4:               return MBTK_KS_FN+4;
		case GLFW_KEY_F5:               return MBTK_KS_FN+5;
		case GLFW_KEY_F6:               return MBTK_KS_FN+6;
		case GLFW_KEY_F7:               return MBTK_KS_FN+7;
		case GLFW_KEY_F8:               return MBTK_KS_FN+8;
		case GLFW_KEY_F9:               return MBTK_KS_FN+9;
		case GLFW_KEY_F10:              return MBTK_KS_FN+10;
		case GLFW_KEY_F11:              return MBTK_KS_FN+11;
		case GLFW_KEY_F12:              return MBTK_KS_FN+12;
		case GLFW_KEY_F13:              return MBTK_KS_FN+13;
		case GLFW_KEY_F14:              return MBTK_KS_FN+14;
		case GLFW_KEY_F15:              return MBTK_KS_FN+15;
		case GLFW_KEY_F16:              return MBTK_KS_FN+16;
		case GLFW_KEY_F17:              return MBTK_KS_FN+17;
		case GLFW_KEY_F18:              return MBTK_KS_FN+18;
		case GLFW_KEY_F19:              return MBTK_KS_FN+19;
		case GLFW_KEY_F20:              return MBTK_KS_FN+20;
		case GLFW_KEY_F21:              return MBTK_KS_FN+21;
		case GLFW_KEY_F22:              return MBTK_KS_FN+22;
		case GLFW_KEY_F23:              return MBTK_KS_FN+23;
		case GLFW_KEY_F24:              return MBTK_KS_FN+24;
		case GLFW_KEY_F25:              return MBTK_KS_FN+25;

		case GLFW_KEY_HOME:             return MBTK_KS_HOME;
		case GLFW_KEY_END:              return MBTK_KS_END;
		case GLFW_KEY_LEFT:             return MBTK_KS_LEFT;
		case GLFW_KEY_RIGHT:            return MBTK_KS_RIGHT;
		case GLFW_KEY_UP:               return MBTK_KS_UP;
		case GLFW_KEY_DOWN:             return MBTK_KS_DOWN;
		case GLFW_KEY_PAGE_UP:          return MBTK_KS_PGUP;
		case GLFW_KEY_PAGE_DOWN:        return MBTK_KS_PGDN;

		case GLFW_KEY_LEFT_SHIFT:       return MBTK_KS_SHIFT_L;
		case GLFW_KEY_RIGHT_SHIFT:      return MBTK_KS_SHIFT_R;
		case GLFW_KEY_LEFT_ALT:         return MBTK_KS_ALT_L;
		case GLFW_KEY_RIGHT_ALT:        return MBTK_KS_ALT_R;
		case GLFW_KEY_LEFT_CONTROL:     return MBTK_KS_CTRL_L;
		case GLFW_KEY_RIGHT_CONTROL:    return MBTK_KS_CTRL_R;

		case GLFW_KEY_DELETE:           return MBTK_KS_DEL;
		case GLFW_KEY_INSERT:           return MBTK_KS_INS;

		/* TODO: this ignores numlock; GLFW_MOD_NUM_LOCK */
		case GLFW_KEY_KP_DECIMAL:       (*mod) |= MBTK_KM_KEYPAD; return numlock ? '.' : MBTK_KS_DEL;
		case GLFW_KEY_KP_0:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '0' : MBTK_KS_INS;
		case GLFW_KEY_KP_1:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '1' : MBTK_KS_END;
		case GLFW_KEY_KP_2:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '2' : MBTK_KS_DOWN;
		case GLFW_KEY_KP_3:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '3' : MBTK_KS_PGDN;
		case GLFW_KEY_KP_4:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '4' : MBTK_KS_LEFT;
		case GLFW_KEY_KP_5:             (*mod) |= MBTK_KM_KEYPAD; return '5';
		case GLFW_KEY_KP_6:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '6' : MBTK_KS_RIGHT;
		case GLFW_KEY_KP_7:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '7' : MBTK_KS_HOME;
		case GLFW_KEY_KP_8:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '8' : MBTK_KS_UP;
		case GLFW_KEY_KP_9:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '9' : MBTK_KS_PGUP;
		case GLFW_KEY_KP_DIVIDE:        (*mod) |= MBTK_KM_KEYPAD; return '/';
		case GLFW_KEY_KP_MULTIPLY:      (*mod) |= MBTK_KM_KEYPAD; return '*';
		case GLFW_KEY_KP_SUBTRACT:      (*mod) |= MBTK_KM_KEYPAD; return '-';
		case GLFW_KEY_KP_ADD:           (*mod) |= MBTK_KM_KEYPAD; return '+';
		case GLFW_KEY_KP_EQUAL:         (*mod) |= MBTK_KM_KEYPAD; return '=';

	}

	return MBTK_KS_INVALID;
}

static void mbtk_glfw_key_xlate(mbtk_event_t *dst, int key, int mods)
{
	dst->data.key.mods = mbtk_glfw_key_mods(mods);
	dst->data.key.sym = mbtk_glfw_key_sym(key, mods, &dst->data.key.mods);
}

