/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

static char mbtk_sdl2_mouse_button_xlate(int button)
{
	switch(button) {
		case SDL_BUTTON_LEFT:    return MBTK_MB_LEFT;
		case SDL_BUTTON_MIDDLE:  return MBTK_MB_MIDDLE;
		case SDL_BUTTON_RIGHT:   return MBTK_MB_RIGHT;
		case SDL_BUTTON_X1:      return MBTK_MB_EXTRA1;
		case SDL_BUTTON_X2:      return MBTK_MB_EXTRA2;
	}
	return MBTK_MB_UNKNOWN;
}

#include "../common/utf8.h"

static mbtk_keymod_t mbtk_sdl2_key_mods(unsigned int mod)
{
	mbtk_keymod_t res = 0;
	if (mod & KMOD_SHIFT)  res |= MBTK_KM_SHIFT;
	if (mod & KMOD_ALT)    res |= MBTK_KM_ALT;
	if (mod & KMOD_CTRL)   res |= MBTK_KM_CTRL;
	return res;
}

static mbtk_keysym_t mbtk_sdl2_key_sym(SDL_Keycode kc, mbtk_keymod_t *mod, int sdlmod)
{
	int numlock = !!(sdlmod & KMOD_NUM);

	if ((kc >= 32) && (kc < 127))
		return kc;

	switch(kc) {
		case SDLK_BACKSPACE:        return MBTK_KS_BACKSPACE;
		case SDLK_TAB:              return MBTK_KS_TAB;
		case SDLK_KP_ENTER:         (*mod) |= MBTK_KM_KEYPAD; return MBTK_KS_ENTER;
		case SDLK_RETURN:           return MBTK_KS_ENTER;
		case SDLK_RETURN2:          return MBTK_KS_ENTER;
		case SDLK_ESCAPE:           return MBTK_KS_ESC;

		case SDLK_F1:               return MBTK_KS_FN+1;
		case SDLK_F2:               return MBTK_KS_FN+2;
		case SDLK_F3:               return MBTK_KS_FN+3;
		case SDLK_F4:               return MBTK_KS_FN+4;
		case SDLK_F5:               return MBTK_KS_FN+5;
		case SDLK_F6:               return MBTK_KS_FN+6;
		case SDLK_F7:               return MBTK_KS_FN+7;
		case SDLK_F8:               return MBTK_KS_FN+8;
		case SDLK_F9:               return MBTK_KS_FN+9;
		case SDLK_F10:              return MBTK_KS_FN+10;
		case SDLK_F11:              return MBTK_KS_FN+11;
		case SDLK_F12:              return MBTK_KS_FN+12;
		case SDLK_F13:              return MBTK_KS_FN+13;
		case SDLK_F14:              return MBTK_KS_FN+14;
		case SDLK_F15:              return MBTK_KS_FN+15;
		case SDLK_F16:              return MBTK_KS_FN+16;
		case SDLK_F17:              return MBTK_KS_FN+17;
		case SDLK_F18:              return MBTK_KS_FN+18;
		case SDLK_F19:              return MBTK_KS_FN+19;
		case SDLK_F20:              return MBTK_KS_FN+20;
		case SDLK_F21:              return MBTK_KS_FN+21;
		case SDLK_F22:              return MBTK_KS_FN+22;
		case SDLK_F23:              return MBTK_KS_FN+23;
		case SDLK_F24:              return MBTK_KS_FN+24;

		case SDLK_HOME:             return MBTK_KS_HOME;
		case SDLK_END:              return MBTK_KS_END;
		case SDLK_LEFT:             return MBTK_KS_LEFT;
		case SDLK_RIGHT:            return MBTK_KS_RIGHT;
		case SDLK_UP:               return MBTK_KS_UP;
		case SDLK_DOWN:             return MBTK_KS_DOWN;
		case SDLK_PAGEUP:           return MBTK_KS_PGUP;
		case SDLK_PAGEDOWN:         return MBTK_KS_PGDN;

		case SDLK_LSHIFT:           return MBTK_KS_SHIFT_L;
		case SDLK_RSHIFT:           return MBTK_KS_SHIFT_R;
		case SDLK_LALT:             return MBTK_KS_ALT_L;
		case SDLK_RALT:             return MBTK_KS_ALT_R;
		case SDLK_LCTRL:            return MBTK_KS_CTRL_L;
		case SDLK_RCTRL:            return MBTK_KS_CTRL_R;

		case SDLK_KP_PERIOD:        (*mod) |= MBTK_KM_KEYPAD; return numlock ? '.' : MBTK_KS_DEL;
		case SDLK_DELETE:           return MBTK_KS_DEL;
		case SDLK_INSERT:           return MBTK_KS_INS;

		case SDLK_KP_0:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '0' : MBTK_KS_INS;
		case SDLK_KP_1:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '1' : MBTK_KS_END;
		case SDLK_KP_2:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '2' : MBTK_KS_DOWN;
		case SDLK_KP_3:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '3' : MBTK_KS_PGDN;
		case SDLK_KP_4:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '4' : MBTK_KS_LEFT;
		case SDLK_KP_5:             (*mod) |= MBTK_KM_KEYPAD; return '5';
		case SDLK_KP_6:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '6' : MBTK_KS_RIGHT;
		case SDLK_KP_7:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '7' : MBTK_KS_HOME;
		case SDLK_KP_8:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '8' : MBTK_KS_UP;
		case SDLK_KP_9:             (*mod) |= MBTK_KM_KEYPAD; return numlock ? '9' : MBTK_KS_PGUP;
		case SDLK_KP_PLUS:          (*mod) |= MBTK_KM_KEYPAD; return '+';
		case SDLK_KP_MINUS:         (*mod) |= MBTK_KM_KEYPAD; return '-';
		case SDLK_KP_MULTIPLY:      (*mod) |= MBTK_KM_KEYPAD; return '*';
		case SDLK_KP_DIVIDE:        (*mod) |= MBTK_KM_KEYPAD; return '/';
	}

	return MBTK_KS_INVALID;
}

static void mbtk_sdl2_key_xlate(mbtk_event_t *dst, SDL_Event *ev)
{
	SDL_KeyboardEvent *kev = &ev->key;

	dst->data.key.scancode = kev->keysym.scancode;
	dst->data.key.mods = mbtk_sdl2_key_mods(kev->keysym.mod);
	dst->data.key.sym = mbtk_sdl2_key_sym(kev->keysym.sym, &dst->data.key.mods, kev->keysym.mod);
}

static void mbtk_sdl2_text_xlate(mbtk_event_t *dst, SDL_Event *ev)
{
	long res = utf8_decode_first(ev->text.text);
	dst->data.key.edit = (res > 0) ? res : 0;
}
