/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <SDL.h>
#include <SDL_thread.h>
#include <SDL_mutex.h>
#include <libmbtk/config.h>
#include <libmbtk/backend.h>
#include <genlist/gendlist.h>

#include "font.h"

typedef struct sdl2_win_s sdl2_win_t;

typedef struct sdl2_display_s {
	mbtk_sdl2_ttfcache_t font_cache;
	double base_time;
	struct {
		/* output */
		int fd;
		unsigned readable:1;
		unsigned writable:1;
		unsigned closed:1;

		/* thread */
		SDL_Thread *tid;
		SDL_sem *handled;       /* main to thread: 0: thread's waiting non-blocking after a read; when main finishes picking up it increases it back to 1 to start the next read */
		int ctrlw, ctrlr;       /* control pipe fds; w is write from main, r is read on the thread's side */
		SDL_Event ev;
	} iot_fdwatch;
} sdl2_display_t;

typedef struct sdl2_win_s {
	SDL_Window *window;
	SDL_Renderer *renderer;
	SDL_Color clr;
	mbtk_sdl2_font_ctx_t *fonts;
	mbtk_sdl2_font_t *font; /* current font */
	sdl2_win_t *last_mouse_subw;

	gdl_list_t subwins;
	gdl_elem_t link;
	unsigned skip_render:1; /* do not render anything in this window (caller will render) */
	unsigned subwin:1;
} sdl2_win_t;
