/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_CALLBACK_H
#define LIBMBTK_CALLBACK_H

/* internal: per widget callbacks, used by the lib implementation  */

#include <libmbtk/config.h>
#include <libmbtk/global_typedefs.h>


typedef struct mbtk_callback_s {
	union {
		mbtk_event_handled_t (*noarg)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data);
		mbtk_event_handled_t (*event)(mbtk_widget_t *w, mbtk_kw_t id, mbtk_event_t *ev, void *user_data);
	} func;
	void *user_data;
	unsigned inhibit:1;
} mbtk_callback_t;



/* Removes all callbacks from a widget */
void mbtk_callback_uninit(mbtk_widget_t *w);

/*** Implementation ***/

typedef mbtk_kw_t mbtk_htcb_key_t;
typedef mbtk_callback_t mbtk_htcb_value_t;

int mbtk_htcb_keyeq(mbtk_htcb_key_t a, mbtk_htcb_key_t b);
unsigned mbtk_htcb_keyhash(mbtk_htcb_key_t k);

#define HT(x) mbtk_htcb_ ## x
#include <libmbtk/src_3rd/genht/ht.h>
#undef HT

#endif
