/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_CURSOR_H
#define LIBMBTK_CURSOR_H

/* API: mouse cursor shape */

#include <libmbtk/libmbtk.h>
#include <libmbtk/global_typedefs.h>

/* Create a mouse cursor from a predefined, named bitmap; returns NULL on error. */
mbtk_mocu_t *mbtk_mocu_create_named(mbtk_display_t *disp, mbtk_mocu_named_t name);

/* Create a mouse cursor from an 1-bit pixmap. Width must be divisible by 8.
   Pixels are drawn where px 0=black, 1=white while mask is 1; if mask is 0,
   the pixel is transparent and px should be 0.
   (Only black&white cursors can be created this way) */
mbtk_mocu_t *mbtk_mocu_create_pixmap(mbtk_display_t *disp, int width, int height, unsigned const char *px, unsigned const char *mask, int hot_x, int hot_y);

/* Free memory allocated for curs and unregister it from the backend */
void mbtk_mocu_free_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs);

/* Set current cursor to curs; if curs is NULL, set it to system default */
void mbtk_mocu_set_cursor(mbtk_display_t *disp, mbtk_mocu_t *curs);


#endif
