/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_DISPLAY_H
#define LIBMBTK_DISPLAY_H

/* API: display is where pixels are drawn to and user input are read from */

#include <libmbtk/global_typedefs.h>
#include <libmbtk/event.h>
#include <libmbtk/src_3rd/genlist/gendlist.h>
#include <libmbtk/src_3rd/genht/htip.h>
#include <libmbtk/src_3rd/genvector/vtp0.h>

struct mbtk_display_s {
	union { void *ptr[64]; long int i[64]; } backend_data; /* opaque backend data */
	mbtk_backend_t *be;

	mbtk_window_t *topwin;
	gdl_list_t windows;          /* all windows open within this display */
	unsigned need_win_redraw:1;  /* if any of the windows needs a window redraw */
	double win_next_redraw;      /* when to next redraw windows (0 means default timeout) */

	const mbtk_style_t *style;   /* style implementation to use for drawing widgets that don't have their own style */
	mbtk_styleprop_t *styleprop; /* fallback (default) style properties for all widgets without own styleprop */

	unsigned short button_mask; /* mouse button mask: each bit is the last known state of a mouse button */

	struct {
		mbtk_event_t ev_win_resize, ev_win_expose;
		double exp_win_resize, exp_win_expose; /* when ev_win_* will next expire */
		double last_win_resize, last_win_expose; /* when ev_win_* was last done */
	} evi; /* event handler internals */

	mbtk_font_id_t default_fid; /* labels are drawn with this font by default */
	htip_t mpm; /* mpm; key is "name" (mbtk_kw_t), value is (mbtk_mbm_styled_t *); see mbm.h */

	gdl_elem_t link;    /* all displays within a backend */
	vtp0_t timers;      /* all timers that are still allocated */
	void *timerq;       /* timer queue for currently running timers */

#if 0
	/* last popup is the topmost */
	int *popups;
	int popup_used, popup_alloced;
#endif
};
#endif

/* Change the top window of display. By default the first window created
   under a display becomes the top window. Top window is used when reporting
   generic, window-idependent events (such as timers, fdwatch, stdin, etc.) */
void mbtk_display_set_topwin(mbtk_display_t *disp, mbtk_window_t *win);

/* Query backend and return bitfield of keyboard modifiers currently pressed */
mbtk_keymod_t mbtk_display_query_kbd_mods(mbtk_display_t *disp);

/*** Internal use ***/

#define mbtk_display_set_mouse_button(disp, button, pressed) \
	do { \
		if (pressed) \
			(disp)->button_mask |= 1 << (button); \
		else \
			(disp)->button_mask &= ~(1 << (button)); \
	} while(0) \

#define mbtk_display_get_mouse_button(disp, button) \
	(!!((disp)->button_mask & (1 << (button))))

/* Redraw all windows with pending layout or draw dirty bits */
void mbtk_display_redraw_windows(mbtk_display_t *disp);
