/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdlib.h>
#include <string.h>
#include <genht/htsi.h>
#include <genht/hash.h>
#include <assert.h>
#include "keywords.h"
#include "util.h"

static htsi_t *mbtk_keywords = NULL;
static mbtk_kw_t mbtk_kw_next = 1;

mbtk_kw_t mbtk_kw(const char *name)
{
	mbtk_kw_t kw;

	if (mbtk_keywords == NULL) {
		mbtk_keywords = htsi_alloc(strhash, strkeyeq);
		assert(mbtk_keywords != NULL);
	}
	kw = htsi_get(mbtk_keywords, (char *)name);
	if (kw == mbtk_kw_invalid) {
		kw = mbtk_kw_next++;
		htsi_set(mbtk_keywords, mbtk_strdup(name), kw);
	}
	return kw;
}

mbtk_kw_t mbtk_kw_uniq(void)
{
	return mbtk_kw_next++;
}

void mbtk_kw_uninit(void)
{
	htsi_entry_t *e;
	for (e = htsi_first(mbtk_keywords); e; e = htsi_next(mbtk_keywords, e))
		free(e->key);

	htsi_free(mbtk_keywords);
	mbtk_keywords = NULL;
}
