/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdlib.h>
#include <string.h>
#include "style.h"
#include "libmbtk.h"
#include "display.h"
#include "keysym.h"

static gdl_list_t mbtk_active_be;
mbtk_font_id_t mbtk_invalid_font_id;

static int init_once = 0;
int mbtk_be_init(mbtk_backend_t *be)
{
	int res;

	if (!init_once) {
		mbtk_style_init();
		mbtk_invalid_font_id.l = -1;
		init_once = 1;
	}

	memset(&be->displays, 0, sizeof(be->displays));

	if (be->be_init != NULL)
		res = be->be_init(be);
	else
		res = 0;

	if (res == 0)
		gdl_append(&mbtk_active_be, be, link);

	return res;
}

void mbtk_be_uninit(mbtk_backend_t *be)
{
	mbtk_display_t *d;

	for(d = gdl_first(&be->displays); d != NULL; d = gdl_first(&be->displays))
		mbtk_display_uninit(be, d);

	gdl_remove(&mbtk_active_be, be, link);
	if (be->be_uninit != NULL)
		be->be_uninit(be);
}


void mbtk_uninit()
{
	mbtk_backend_t *be;

	for(be = gdl_first(&mbtk_active_be); be != NULL; be = gdl_first(&mbtk_active_be))
		mbtk_be_uninit(be);

	mbtk_keysym_uninit();
	mbtk_kw_uninit();
}

