/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_H
#define LIBMBTK_H

/* API:  global resources; intended only for applications */

#include <libmbtk/backend.h>
#include <libmbtk/draw.h>
#include <libmbtk/event.h>
#include <libmbtk/window.h>

/* One time backend init, per process, per backend */
int mbtk_be_init(mbtk_backend_t *be);
void mbtk_be_uninit(mbtk_backend_t *be);

/* Per display init/uninit; call when backend is inited */
int mbtk_display_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char **argv[]);
void mbtk_display_uninit(mbtk_backend_t *be, mbtk_display_t *disp);


/* Clipboard is per display and string only. clipboard_get() returns NULL or a
   string that needs to be free'd using clipboard_free(). */
void mbtk_clipboard_set(mbtk_display_t *display, const char *new_content);
char *mbtk_clipboard_get(mbtk_display_t *display);
void mbtk_clipboard_free(mbtk_display_t *display, char *str_got);

/* Return time elapsed from display init, in seconds. Resolution is backend
   dependent but is at least millisec. */
double mbtk_get_time_el(mbtk_display_t *display);

/* Call this to uninitialize all backends and close all open displays and
   windows free all memory used by the library. */
void mbtk_uninit();

#endif
