/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "util.h"
#include "config.h"

char *mbtk_strdup(const char *s)
{
	int l = strlen(s);
	char *o = malloc(l+1);
	memcpy(o, s, l+1);
	return o;
}

void mbtk_error(const char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, "MBTK ERROR: ");
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
}

void mbtk_warning(const char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, "MBTK warning: ");
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
}

#ifndef MBTK_HAVE_SNPRINTF
#include <stdarg.h>
#include <stdio.h>
/* Assumes the caller will never render strings longer than 1k */
int mbtk_snprintf_(char *dst, size_t size, const char *fmt, ...)
{
	char buff[1024];
	size_t l;
	va_list ap;
	
	va_start(ap, fmt);
	l = vsprintf(buff, fmt, ap);
	if (l >= 1024) {
		fprintf(stderr, "mbtk_snprintf_(): string too long\n");
		abort();
	}
	va_end(ap);

	if (size <= l)
		l = size-1;

	buff[l] = '\0';
	memcpy(dst, buff, l+1);
	return l;
}
#endif
