/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/Common.h>


MBTK_INLINE int mbtk_comm_frame_fallback_minsize_(mbtk_widget_t *w, long *mx, long *my, mbtk_widget_flags_t flags)
{
	long l, r, t, b;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	w->hvbox.fr_l = l; w->hvbox.fr_r = r;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r; *my = t+b;

	return 0;
}

int mbtk_comm_frame_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	return mbtk_comm_frame_fallback_minsize_(w, mx, my, w->flags);
}

int mbtk_comm_frame_fallback_minsize_inactive(mbtk_widget_t *w, long *mx, long *my)
{
	mbtk_widget_flags_t flags = w->flags;
	flags.activated = 0;
	return mbtk_comm_frame_fallback_minsize_(w, mx, my, flags);
}

MBTK_INLINE void mbtk_comm_frame_fallback_draw_(mbtk_widget_t *w, mbtk_widget_flags_t flags)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	style->draw_bg_rect(win, sp, flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
	style->draw_box_frame(win, sp, flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
}

void mbtk_comm_frame_fallback_draw(mbtk_widget_t *w)
{
	mbtk_comm_frame_fallback_draw_(w, w->flags);
}

void mbtk_comm_frame_fallback_draw_inactive(mbtk_widget_t *w)
{
	mbtk_widget_flags_t flags = w->flags;
	flags.activated = 0;
	mbtk_comm_frame_fallback_draw_(w, flags);
}


void mbtk_comm_frame_fallback_draw_overlay(mbtk_widget_t *w)
{
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);

	style->draw_box_overlay(win, sp, w->flags, w->hvbox.x, w->hvbox.y, w->hvbox.w, w->hvbox.h);
}

mbtk_widget_t *mbtk_comm_embed1_set_child(mbtk_widget_t *parent, mbtk_widget_t *w)
{
	mbtk_widget_t *res = NULL;

	if (parent->hvbox.ch_first != NULL) {
		res = (mbtk_widget_t *)parent->hvbox.ch_first;
		mbtk_widget_unembed((mbtk_widget_t *)parent->hvbox.ch_first);
	}
	hvbox_append_existing_box(&parent->hvbox, &w->hvbox);

	/* for the case parent button is already added */
	mbtk_widget_embedded(parent, w);
	return res;
}

mbtk_widget_t *mbtk_comm_embed_replace(mbtk_widget_t *host, mbtk_widget_t *old, mbtk_widget_t *new)
{
	mbtk_widget_t *parent = (mbtk_widget_t *)old->hvbox.parent;
	hvbox_append_existing_box_at(&old->hvbox, &new->hvbox);
	mbtk_widget_unembed(old);
	mbtk_widget_embedded(parent, new);
	return old;
}

