/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/widget/button.h>
#include <libmbtk/widget/label.h>
#include <libmbtk/widget/Common.h>
#include <libmbtk/widget/hand_pushb.h>


static void mbtk_button_imp_destroy(mbtk_widget_t *w) { }

static const mbtk_widget_imp_t mbtk_button_imp = {
	mbtk_button_imp_destroy,

	mbtk_comm_frame_fallback_minsize,
	mbtk_comm_frame_fallback_draw,
	mbtk_comm_frame_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	NULL  /* pre_child_unlink */
};

mbtk_button_t *mbtk_button_new(mbtk_button_t *caller_alloced)
{
	mbtk_button_t *button;
	mbtk_kw_t kw_button;

	mbtk_kw_cache(kw_button, "button");

	mbtk_create_wt(button, kw_button, &mbtk_button_imp, caller_alloced);
	button->w.hvbox.type = HVBOX_HBOX;
	mbtk_handler_pushb_install(&button->w);
	return button;
}

mbtk_button_t *mbtk_button_new_with_widget(mbtk_button_t *caller_alloced, mbtk_widget_t *child)
{
	mbtk_button_t *res = mbtk_button_new(NULL);
	mbtk_button_set_child(res, child);
	return res;
}

mbtk_button_t *mbtk_button_new_with_label(const char *str)
{
	return mbtk_button_new_with_widget(NULL, (mbtk_widget_t *)mbtk_label_new(NULL, str));
}


mbtk_widget_t *mbtk_button_set_child(mbtk_button_t *parent, mbtk_widget_t *w)
{
	return mbtk_comm_embed1_set_child(&parent->w, w);
}

void mbtk_button_callback_click(mbtk_button_t *b, mbtk_event_handled_t (*cb)(mbtk_widget_t *w, mbtk_kw_t id, void *user_data), void *user_data)
{
	mbtk_kw_t kw_clicked;
	mbtk_kw_cache(kw_clicked, "clicked");
	mbtk_callback_set_noarg(&b->w, kw_clicked, cb, user_data);
}

