/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <libmbtk/global_typedefs.h>
#include <libmbtk/handler.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>

static const char my_name[] = "hand_pushb";

typedef struct mbtk_pushb_handler_s {
	mbtk_handler_t hdr;
} mbtk_pushb_handler_t;

static mbtk_pushb_handler_t pushb_handler;
static int pushb_handler_inited = 0;

static void pushb_fire_cb(mbtk_widget_t *widget)
{
	mbtk_kw_t kw_clicked;

	mbtk_kw_cache(kw_clicked, "clicked");
	mbtk_callback_noarg(widget, kw_clicked);
}

static mbtk_event_handled_t pushb_event(mbtk_widget_t *widget, mbtk_event_t *ev)
{
	switch(ev->type) {
		case MBTK_EV_KEY_PRESS:
			if (ev->data.key.sym == MBTK_KS_ENTER) {
				mbtk_widget_set_activated(widget, 1);
				pushb_fire_cb(widget);
				return MBTK_EVENT_HANDLED;
			}
			break;
		case MBTK_EV_KEY_RELEASE:
			if (ev->data.key.sym == MBTK_KS_ENTER) {
				mbtk_widget_set_activated(widget, 0);
				return MBTK_EVENT_HANDLED;
			}
			break;
		case MBTK_EV_MOUSE_PRESS:
			if (ev->data.mouse.button == MBTK_MB_LEFT) {
				mbtk_window_set_focus(widget->win, widget, 1);
				mbtk_widget_set_activated(widget, 1);
				widget->win->origin = widget;
				return MBTK_EVENT_HANDLED;
			}
			break;
		case MBTK_EV_MOUSE_RELEASE:
			if (ev->data.mouse.button == MBTK_MB_LEFT) {
				if (widget->flags.activated)
					pushb_fire_cb(widget);
				mbtk_widget_set_activated(widget, 0);
				return MBTK_EVENT_HANDLED;
			}
			break;

		case MBTK_EV_FOCUS_ENTER:
			mbtk_widget_set_activated(widget, (widget == widget->win->origin) && mbtk_display_get_mouse_button(widget->win->display, MBTK_MB_LEFT));
			return MBTK_EVENT_HANDLED;
		case MBTK_EV_FOCUS_LEAVE:
			mbtk_widget_set_activated(widget, 0);
			return MBTK_EVENT_HANDLED;

		default:;
	}
	return MBTK_EVENT_NOT_HANDLED;
}


void mbtk_handler_pushb_install(mbtk_widget_t *widget)
{
	if (widget->handler != NULL)
		mbtk_widget_del_handler(widget);

	if (!pushb_handler_inited) {
		pushb_handler.hdr.name = my_name;
		pushb_handler.hdr.event = pushb_event;
		pushb_handler_inited = 1;
	}
	widget->handler = &pushb_handler.hdr;
}
