/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <math.h>
#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/util.h>
#include <libmbtk/widget/pane.h>
#include <libmbtk/widget/box.h>
#include <libmbtk/widget/pane_sep.h>
#include <libmbtk/widget/hand_range.h>
#include <libmbtk/widget/Common.h>


static void mbtk_pane_imp_destroy(mbtk_widget_t *w) { }

static void mbtk_pane_post_update_coords(hvbox_t *box)
{
	mbtk_pane_t *pane = (mbtk_pane_t *)box;
	long target, c, maxsize;
	long size = pane->horiz ? pane->w.hvbox.w : pane->w.hvbox.h;
	hvbox_t *ch1 = pane->w.hvbox.ch_first, *sep = ch1->next, *ch2 = sep->next;

	if (pane->target_percent > 0) {
		if (pane->target_percent > 1)
			pane->target_percent = 1;
		target = (double)size * pane->target_percent;
	}
	else
		target = pane->target_pixel;


	if (pane->horiz) {
		c = box->x + box->padding + box->fr_l;

		maxsize = box->w - ch2->w_min_calc - sep->w - box->padding - 2 * box->spacing - box->fr_r;

		ch1->w = MBTK_MAX(target, ch1->w_min_calc); /* minimum is ch1's minsize */
		ch1->w = MBTK_MIN(ch1->w, maxsize);         /* maximum depends on ch2 and sep */
		c += ch1->w + (ch1->hidden ? 0 : box->spacing);

		sep->x = c;
		c += sep->w + (sep->hidden ? 0 : box->spacing);

		ch2->x = c;
		ch2->w = box->w - ch2->x - box->padding - box->fr_r;
	}
	else {
		c = box->y + box->padding + box->fr_t;

		maxsize = box->h - ch2->h_min_calc - sep->h - box->padding - 2 * box->spacing - box->fr_b;

		ch1->h = MBTK_MAX(target, ch1->h_min_calc); /* minimum is ch1's minsize */
		ch1->h = MBTK_MIN(ch1->h, maxsize);         /* maximum depends on ch2 and sep */
		c += ch1->h + (ch1->hidden ? 0 : box->spacing);

		sep->y = c;
		c += sep->h + (sep->hidden ? 0 : box->spacing);

		ch2->y = c;
		ch2->h = box->h - ch2->y - box->padding - box->fr_b;
	}

	hvbox_layout_update_after_resize(ch1);
	hvbox_layout_update_after_resize(ch2);
}

static int mbtk_pane_get_propa(mbtk_widget_t *w, mbtk_kw_t key, mbtk_arg_t *dst, int arrlen)
{
	mbtk_kw_t kw_range_val, kw_range_coords, kw_range_horiz;
	mbtk_pane_t *pane = (mbtk_pane_t *)w;
	hvbox_t *ch1 = pane->w.hvbox.ch_first, *sep = ch1->next;

	mbtk_kw_cache(kw_range_val, "range.val");
	mbtk_kw_cache(kw_range_coords, "range.coords");
	mbtk_kw_cache(kw_range_horiz, "range.horiz");

	if ((key == kw_range_val) && (arrlen == 4)) {
		dst[0].type = MBTK_DOUBLE; dst[0].v.d = 0.0;
		dst[1].type = MBTK_DOUBLE; dst[1].v.d = 1.0;
		if (pane->horiz) {
			dst[2].type = MBTK_DOUBLE; dst[2].v.d = (double)sep->x / (double)pane->w.hvbox.w;
			dst[3].type = MBTK_DOUBLE; dst[3].v.d = ((double)sep->x + (double)sep->w) / (double)pane->w.hvbox.w;
		}
		else {
			dst[2].type = MBTK_DOUBLE; dst[2].v.d = (double)sep->y / (double)pane->w.hvbox.h;
			dst[3].type = MBTK_DOUBLE; dst[3].v.d = ((double)sep->y + (double)sep->h) / (double)pane->w.hvbox.h;
		}
		return 4;
	}


	if ((key == kw_range_coords) && (arrlen == 4)) {
		if (pane->horiz) {
			dst[0].type = MBTK_DOUBLE; dst[0].v.c.x = pane->w.hvbox.x;
			dst[1].type = MBTK_DOUBLE; dst[1].v.c.x = pane->w.hvbox.x + pane->w.hvbox.w;
			dst[2].type = MBTK_DOUBLE; dst[2].v.c.x = sep->x;
			dst[3].type = MBTK_DOUBLE; dst[3].v.c.x = sep->x + sep->w;
		}
		else {
			dst[0].type = MBTK_DOUBLE; dst[0].v.c.y = pane->w.hvbox.y;
			dst[1].type = MBTK_DOUBLE; dst[1].v.c.y = pane->w.hvbox.y + pane->w.hvbox.h;
			dst[2].type = MBTK_DOUBLE; dst[2].v.c.y = sep->y;
			dst[3].type = MBTK_DOUBLE; dst[3].v.c.y = sep->y + sep->h;
		}

		return 4;
	}


	if ((key == kw_range_horiz) && (arrlen == 1)) {
		dst[0].type = MBTK_BOOL; dst[0].v.b = pane->horiz;
		return 1;
	}

	return -1;
}

static int mbtk_pane_set_pos_px(mbtk_pane_t *pane, long px)
{
	pane->target_percent = -1;
	pane->target_pixel = px;
	mbtk_widget_changed(&pane->w, 1);
	return 0;
}

static int mbtk_pane_set_pos_percent(mbtk_pane_t *pane, double percent)
{
	pane->target_percent = percent;
	pane->target_pixel = -1;
	mbtk_widget_changed(&pane->w, 1);
	return 0;
}

static int mbtk_pane_set_propa(mbtk_widget_t *w, mbtk_kw_t key, const mbtk_arg_t *src, int arrlen)
{
	mbtk_kw_t kw_range_val, kw_range_coords;
	mbtk_pane_t *pane = (mbtk_pane_t *)w;

	mbtk_kw_cache(kw_range_val, "range.val");
	mbtk_kw_cache(kw_range_coords, "range.coords");

	if ((key == kw_range_val) && (arrlen == 4)) {
		if (src[2].type != MBTK_DOUBLE)
			return -1;

		return mbtk_pane_set_pos_percent(pane, src[2].v.d);
	}

	if ((key == kw_range_coords) && (arrlen == 4)) {
		if (src[2].type != MBTK_DOUBLE)
			return -1;

		return mbtk_pane_set_pos_px(pane, pane->horiz ? src[2].v.c.x : src[2].v.c.y);
	}

	return -1;
}

static void mbtk_pane_pre_child_unlink(mbtk_widget_t *w, mbtk_widget_t *child)
{
	mbtk_pane_t *pane = (mbtk_pane_t *)w;
	hvbox_t *ch1, *sep, *ch2;

	if (w->destroying || pane->child_unlink_lock)
		return;

	ch1 = pane->w.hvbox.ch_first;
	sep = ch1->next;
	ch2 = sep->next;

	if (child == (mbtk_widget_t *)sep) {
		mbtk_error("mbtk_pane: pane sep destroyed (shouldn't happen)\n");
		return;
	}

	if ((child == &pane->dummy[0].w) || (child == &pane->dummy[1].w)) {
		mbtk_error("mbtk_pane: pane dummy destroyed (shouldn't happen)\n");
		return;
	}

	if (child == (mbtk_widget_t *)ch1) {
		hvbox_insert_existing_box(&pane->w.hvbox, &pane->dummy[0].w.hvbox);
		mbtk_widget_embedded(&pane->w, &pane->dummy[0].w);
	}

	if (child == (mbtk_widget_t *)ch2) {
		hvbox_append_existing_box(&pane->w.hvbox, &pane->dummy[1].w.hvbox);
		mbtk_widget_embedded(&pane->w, &pane->dummy[1].w);
	}

}

static const mbtk_widget_imp_t mbtk_pane_imp = {
	mbtk_pane_imp_destroy,

	mbtk_comm_frame_fallback_minsize_inactive,
	mbtk_comm_frame_fallback_draw_inactive,
	NULL, /* draw overlay */

	mbtk_pane_get_propa,
	mbtk_pane_set_propa,

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	mbtk_pane_pre_child_unlink
};

mbtk_pane_t *mbtk_pane_new(mbtk_pane_t *caller_alloced, int horiz)
{
	mbtk_pane_t *pane;
	mbtk_kw_t kw_pane;
	mbtk_box_t *bx = NULL;
	mbtk_pane_sep_t *sep = mbtk_pane_sep_new(NULL, !horiz);
	static int cbs_inited = 0;
	static hvbox_cbs_t cbs;

	if (!cbs_inited) {
		cbs.post_update_coords = mbtk_pane_post_update_coords;
		cbs_inited = 1;
	}

	mbtk_kw_cache(kw_pane, "pane");

	mbtk_create_wt(pane, kw_pane, &mbtk_pane_imp, caller_alloced);
	pane->target_percent = 0.5;
	pane->target_pixel = -1;
	pane->w.hvbox.type = horiz ? HVBOX_HBOX : HVBOX_VBOX;
	pane->horiz = horiz;
	pane->w.hvbox.cbs = &cbs;

	bx = mbtk_box_new(&pane->dummy[0], 1);
	hvbox_append_existing_box(&pane->w.hvbox, &bx->w.hvbox);
	hvbox_append_existing_box(&pane->w.hvbox, &sep->w.hvbox);
	bx = mbtk_box_new(&pane->dummy[1], 1);
	hvbox_append_existing_box(&pane->w.hvbox, &bx->w.hvbox);


	mbtk_handler_range_install(&pane->w);
	mbtk_handler_range_cfg_step(&pane->w, 0);
	mbtk_handler_range_cfg_keys(&pane->w, 0);

	return pane;
}


void mbtk_pane_set_percent(mbtk_pane_t *pane, double curr)
{
	mbtk_pane_set_pos_percent(pane, curr);
}

void mbtk_pane_set_pixel(mbtk_pane_t *pane, long curr)
{
	mbtk_pane_set_pos_px(pane, curr);
}


mbtk_widget_t *mbtk_pane_replace_child(mbtk_pane_t *pane, int idx, mbtk_widget_t *w)
{
	mbtk_widget_t *old, *res;

	switch(idx) {
		case 0: old = (mbtk_widget_t *)pane->w.hvbox.ch_first; break;
		case 1: old = (mbtk_widget_t *)pane->w.hvbox.ch_first->next->next; break;
		default:
			mbtk_error("Invalid pane child index");
			return NULL;
	}

	pane->child_unlink_lock = 1;
	res = mbtk_comm_embed_replace(&pane->w, old, w);
	pane->child_unlink_lock = 0;
	if ((res == &pane->dummy[0].w) || (res == &pane->dummy[1].w))
		return NULL;
	return res;
}

void mbtk_pane_set_child(mbtk_pane_t *pane, int idx, mbtk_widget_t *w)
{
	mbtk_widget_t *old = mbtk_pane_replace_child(pane, idx, w);
	if (old != NULL)
		mbtk_widget_destroy(old);
}


mbtk_widget_t *mbtk_pane_subwidget(mbtk_pane_t *pane, mbtk_kw_t subname)
{
	hvbox_t *ch0 = pane->w.hvbox.ch_first;
	mbtk_kw_t kw_child_0, kw_child_1, kw_sep;

	mbtk_kw_cache(kw_child_0, "child.0");
	mbtk_kw_cache(kw_child_1, "child.1");
	mbtk_kw_cache(kw_sep, "sep");

	if (subname == kw_child_0) return (mbtk_widget_t *)ch0;
	if (subname == kw_sep) return (mbtk_widget_t *)ch0->next;
	if (subname == kw_child_1) return (mbtk_widget_t *)ch0->next->next;

	return NULL;
}
