/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#define MBTK_NO_INCLUDE_WIDGETS
#include <libmbtk/widget.h>
#include <libmbtk/widget_imp.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/widget/separator.h>

static int mbtk_separator_fallback_minsize(mbtk_widget_t *w, long *mx, long *my)
{
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b;

	flags.activated = 1;

	style->min_box_frame_size(w->win, sp, flags, &l, &r, &t, &b);
	w->hvbox.fr_l = t; w->hvbox.fr_r = b;
	w->hvbox.fr_t = t; w->hvbox.fr_b = b;
	*mx = l+r; *my = t+b;

	return 0;
}

static void mbtk_separator_fallback_draw(mbtk_widget_t *w)
{
	mbtk_separator_t *separator = (mbtk_separator_t *)w;
	mbtk_window_t *win = w->win;
	const mbtk_style_t *style = mbtk_widget_style(w);
	mbtk_styleprop_t *sp = mbtk_styleprop_get(w);
	mbtk_widget_flags_t flags = w->flags;
	long l, r, t, b, wi, he;

	flags.activated = 1;
	style->min_box_frame_size(win, sp, flags, &l, &r, &t, &b);

	wi = separator->horiz ? w->hvbox.w : t+b;
	he = separator->horiz ? l+r : w->hvbox.h;

	style->draw_box_frame(win, sp, flags, w->hvbox.x, w->hvbox.y, wi, he);
}

static void mbtk_separator_fallback_draw_overlay(mbtk_widget_t *w)
{
}



static const mbtk_widget_imp_t mbtk_separator_imp = {
	NULL, /* mbtk_separator_destroy */

	mbtk_separator_fallback_minsize,
	mbtk_separator_fallback_draw,
	mbtk_separator_fallback_draw_overlay,

	NULL,  /* get_prop */
	NULL,  /* set_prop */

	NULL, /* post_assign */
	NULL, /* pre_unassign */

	NULL, /* pre_appear */
	NULL, /* post_disappear */

	NULL  /* pre_child_unlink */
};

mbtk_separator_t *mbtk_separator_new(mbtk_separator_t *caller_alloced, int horiz)
{
	mbtk_separator_t *separator;
	mbtk_kw_t kw_hsep, kw_vsep;

	mbtk_kw_cache(kw_hsep, "hsep");
	mbtk_kw_cache(kw_vsep, "vsep");

	mbtk_create_wt(separator, (horiz ? kw_hsep : kw_vsep), &mbtk_separator_imp, caller_alloced);
	separator->w.hvbox.type = (horiz ? HVBOX_HBOX : HVBOX_VBOX);
	separator->horiz = horiz;

	return separator;
}

mbtk_separator_t *mbtk_hsep_new(mbtk_separator_t *caller_alloced) { return mbtk_separator_new(caller_alloced, 1); }
mbtk_separator_t *mbtk_vsep_new(mbtk_separator_t *caller_alloced) { return mbtk_separator_new(caller_alloced, 0); }


