/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef LIBMBTK_WIDGET_TBL_H
#define LIBMBTK_WIDGET_TBL_H

#include <libmbtk/widget.h>
#include <libmbtk/widget_inlines.h>

/* Boxes are used as a layout tool, to arrange widgets and dynamically
   allocate space. Layout widget. */
typedef struct mbtk_tbl_s {
	mbtk_widget_t w;
	unsigned framed:1;        /* draw frame when set */
	unsigned no_background:1; /* do not draw background rectangle when set */
	unsigned bg_even_odd:1;   /* if bg is not disabled, draw it with alternating color per row */
	unsigned bg_first:1;      /* first row has different color (for header); only if bg_even_odd is true */
} mbtk_tbl_t;

mbtk_tbl_t *mbtk_tbl_new(mbtk_tbl_t *caller_alloced, unsigned short num_cols);

/* Prepend or append a widget as a child of the parent widget. The parent
   widget must be a htbl or a vtbl. Returns w on success or NULL on failure.
   Both insert and append are O(1). */
mbtk_widget_t *mbtk_tbl_add_widget(mbtk_tbl_t *parent, mbtk_widget_t *w, int prepend);

/* Insert a widget before or after an existing sibling. The parent
   widget must be a htbl or a vtbl. Returns w on success or NULL on failure.
   NOTE: inserting before (insert=1) is O(n). Append is O(1). */
mbtk_widget_t *mbtk_tbl_add_widget_at(mbtk_widget_t *at, mbtk_widget_t *w, int ins_before);

/* Sets whether the tbl has a frame; unless style code has custom draw
   function for a widget, the fallback draw of a widget typically uses framed
   tbles to draw different sections of the widget. Boxes used for layout purposes
   re unrfamed ("invisible"). The frame widget (a thin frame with a label) is
   a differen widget. */
void mbtk_tbl_set_framed(mbtk_tbl_t *tbl, mbtk_bool_t has_frame);

/* When called with true, make all cols widths or row heights the same
   (matching the widest/tallest col/row, ignoring HVBOX_FILL span) */
void mbtk_tbl_set_homogenous_cols(mbtk_tbl_t *tbl, mbtk_bool_t hom);
void mbtk_tbl_set_homogenous_rows(mbtk_tbl_t *tbl, mbtk_bool_t hom);
void mbtk_tbl_set_homogenous(mbtk_tbl_t *tbl, mbtk_bool_t col, mbtk_bool_t row);

MBTK_WIDGET_INLINES(tbl)

#endif
