/*  miniboxtk - small GUI toolkit/widget system
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <libmbtk/config.h>
#include <stdlib.h>
#include <string.h>
#include <libmbtk/callback_imp.h>
#include <libmbtk/draw.h>

/* API: helpers used in widget implementation */

/* Create a new widget w from an user allocation or by caling calloc() */
#define mbtk_create_wt(widget, type_kw, imps, caller_alloced_) \
	do { \
		if (caller_alloced != NULL) { \
			widget = caller_alloced_; \
			memset(widget, 0, sizeof(*widget)); \
			widget->w.flags.caller_alloced = 1; \
		} \
		else \
			widget = calloc(sizeof(*widget), 1); \
		widget->w.struct_size = sizeof(*widget); \
		widget->w.type = type_kw; \
		widget->w.hvbox.type = HVBOX_WIDGET; \
		widget->w.imp = imps; \
		widget->w.flags.dirty = 1; \
		widget->w.flags.created = 1; \
		widget->w.hvbox.align = mbtk_default_align; \
		widget->w.hvbox.span = mbtk_default_span; \
		widget->w.hvbox.spacing = mbtk_default_spacing; \
		widget->w.hvbox.padding = mbtk_default_padding; \
	} while(0)

void mbtk_widget_minsize(mbtk_widget_t *widget, long *mx, long *my);


/* Call this if previously floating widget got embedded under a parent */
MBTK_INLINE void mbtk_widget_embedded(mbtk_widget_t *parent, mbtk_widget_t *w);

/* Call this to unlink an embedded widget; calls hvbox unlink and unassigns */
MBTK_INLINE void mbtk_widget_unembed(mbtk_widget_t *w);


/* Look up color by kw_clr in w's style and set drawing color for w->win if
   found. Returns 0 on success, -1 if not found */
MBTK_INLINE int mbtk_widget_draw_set_style_color(mbtk_widget_t *w, mbtk_kw_t kw_clr);


/*** Implementation ***/

MBTK_INLINE void mbtk_widget_embedded(mbtk_widget_t *parent, mbtk_widget_t *w)
{

	if (w->win == parent->win)
		return;

	w->win = parent->win;
	if (w->win != NULL)
		mbtk_win_dirty_layout(w->win);

	mbtk_widget_set_win(w, parent->win);
	mbtk_widget_assign(w, parent->render_win);

	mbtk_widget_minsize_recurse(w);
}

MBTK_INLINE void mbtk_widget_unembed(mbtk_widget_t *w)
{
	mbtk_widget_unlink(w);
}

MBTK_INLINE int mbtk_widget_draw_set_style_color(mbtk_widget_t *w, mbtk_kw_t kw_clr)
{
	mbtk_arg_t hica;

	if ((mbtk_widget_style_get_prop(w, kw_clr, &hica) == 0) && (hica.type == MBTK_ULONG)) {
		int r, g, b, a;
		r = (hica.v.ul & 0xff0000000ul) >> 24;
		g = (hica.v.ul & 0x000ff0000ul) >> 16;
		b = (hica.v.ul & 0x00000ff00ul) >> 8;
		a = (hica.v.ul & 0x0000000fful);
		mbtk_draw_color(w->win, r, g, b, a);
		return 0;
	}
	return -1;
}
