/*  miniboxtk - small GUI toolkit/widget system - test programs
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#ifndef MBTK_TESTS_BE_INIT_H
#define MBTK_TESTS_BE_INIT_H

#include <string.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/backend/sdl2.h>
#include <libmbtk/backend/glfw.h>
#include <libmbtk/backend/xlib.h>
#include <libmbtk/backend/test.h>

char *backend_name;


/* Initialize be and disp from $BACKEND and argc/argv and return a font path array
   Exits on error. */
char **tests_backend_init(mbtk_backend_t *be, mbtk_display_t *disp, int *argc, char ***argv)
{
	char **font;
	int res;

	backend_name = getenv("BACKEND");
	if (backend_name == NULL)
		backend_name = "xlib";

#if HAVE_MBTK_BE_xlib
	if (strcmp(backend_name, "xlib") == 0) {
		static char *fnt[] = { "-*-helvetica-*-*-*-*-*-*-*-*-*-*-*-*" };
		font = fnt;
		res = mbtk_xlib_backend(be);
	}
#endif
#if HAVE_MBTK_BE_sdl2
	if (strcmp(backend_name, "sdl2") == 0) {
		static char *fnt[] = { "../../font/Vera.ttf,size=13" };
		font = fnt;
		res = mbtk_sdl2_backend(be);
	}
#endif
#if HAVE_MBTK_BE_glfw
	if (strcmp(backend_name, "glfw") == 0) {
		static char *fnt[] = { "../../font/Vera.ttf,size=13" };
		font = fnt;
		res = mbtk_glfw_backend(be);
	}
#endif
#if HAVE_MBTK_BE_test
	if (strcmp(backend_name, "test") == 0) {
		static char *fnt[] = { "dummy-font,key=val" };
		font = fnt;
		res = mbtk_test_backend(be);
	}
#endif

	if (res != 0) {
		fprintf(stderr, "Unable to set up backend %s\n", backend_name);
		exit(1);
	}

	if (mbtk_be_init(be) != 0) {
		fprintf(stderr, "Unable to initialize mbtk backend %s\n", backend_name);
		exit(1);
	}

	if (mbtk_display_init(be, disp, argc, argv) != 0) {
		fprintf(stderr, "Unable to initialize mbtk display with backend %s\n", backend_name);
		exit(1);
	}

	return font;
}

#endif
