/*  miniboxtk - small GUI toolkit/widget system - test programs
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include "../be_init.h"

static char **font;
static int stay;

static mbtk_event_handled_t button_toggle(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Toggle: '%s' %d\n", (char *)w->user_data, w->flags.activated);
	return 0;
}

static mbtk_event_handled_t button_pre_destroy(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	fprintf(stderr, "Destroy: '%s'\n", (char *)w->user_data);
	free(w->user_data);
	return 0;
}

static void add_button(mbtk_box_t *parent, mbtk_handler_radio_t *handler)
{
	mbtk_tbutton_t *wtb;
	char *labstr;
	static long ctr = 0;

	labstr = malloc(64);
	sprintf(labstr, "tbutton #%ld", ctr++);
	wtb = mbtk_tbutton_new_with_label(labstr);
	wtb->w.user_data = labstr;
	mbtk_tbutton_callback_toggle(wtb, button_toggle, NULL);
	mbtk_tbutton_callback_pre_destroy(wtb, button_pre_destroy, NULL);
	mbtk_handler_radio_add_widget(handler, &wtb->w);
	mbtk_box_add_widget(parent, &wtb->w, 0);
}

static void del_button(mbtk_widget_t *widget)
{
	mbtk_widget_destroy(widget);
}


int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *vbox;
	mbtk_label_t *wl;
	mbtk_handler_radio_t *hr;
	int n;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);


	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk simple widgets", -1, -1, 300, 200);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	vbox = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &vbox->w);
	hr = mbtk_handler_radio_alloc(win);
	wl = mbtk_label_new(NULL, "Press 'a' to add a button,\npress 'd' to remove focused");
	mbtk_box_add_widget(vbox, &wl->w, 0);


	for(n = 0; n < 4; n++)
		add_button(vbox, hr);

	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_iterate(&disp, &ev, 1)) {
/*			printf("ev %d\n", ev.type);*/
			if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;
				if (ev.data.key.sym == 'a')
					add_button(vbox, hr);
				if ((ev.data.key.sym == 'd') && (win->focus != NULL))
					del_button(win->focus);
			}
		}
	}

	/* automatically closes all windows, displays and backends */
	mbtk_uninit();
}
