/*  miniboxtk - small GUI toolkit/widget system - test programs
    Copyright (C) 2022  Tibor 'Igor2' Palinkas

    (Supported by NLnet NGI0 PET Fund in 2022)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://repo.hu/projects/miniboxtk
      contact lead developer: http://www.repo.hu/projects/miniboxtk/contact.html
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libmbtk/libmbtk.h>
#include <libmbtk/widget.h>
#include <libmbtk/debug_inlines.h>
#include <libmbtk/style_f3d.h>
#include <libmbtk/arg_t.h>
#include <libmbtk/mpm.h>
#include "../be_init.h"

static char **font;
static int stay;

static mbtk_event_handled_t scroll_to_0_0(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_scrolled_t *scr = user_data;

	mbtk_scrolled_set_content_origin(scr, 0, 0);

	if (0) {
		mbtk_widget_t *r = (mbtk_widget_t *)scr->subwin->root;
		mbtk_dbg_widget_dump_subtree(stdout, r);
	}
	return 0;
}

static mbtk_event_handled_t scroll_to_20_10(mbtk_widget_t *w, mbtk_kw_t id, void *user_data)
{
	mbtk_scrolled_t *scr = user_data;
	mbtk_scrolled_set_content_origin(scr, 20, 10);
	return 0;
}

int main(int argc, char *argv[])
{
	mbtk_window_t *win;
	mbtk_backend_t be;
	mbtk_display_t disp;
	mbtk_box_t *vbox, *hbox1, *hbox2, *scvbox;
	mbtk_label_t *lab;
	mbtk_scrolled_t *scr;
	mbtk_button_t *btn;

	font = tests_backend_init(&be, &disp, &argc, &argv);
	mbtk_f3d_style(&disp);


	disp.default_fid = mbtk_resolve_font(&disp, 1, font);

	win = mbtk_window_new(&disp, MBTK_WNTY_NORMAL, "miniboxtk odd widgets", -1, -1, 300, 300);
	if (win == NULL) {
		fprintf(stderr, "Unable to create top window\n");
		exit(1);
	}

	vbox = mbtk_vbox_new(NULL);
	mbtk_window_set_root_widget(win, &vbox->w);

	/* add a label to shift scrolled many pixels down to expose coord transformation bugs */
	lab = mbtk_label_new(NULL, "Scrolled subwindow:");
	mbtk_box_add_widget(vbox, &lab->w, 0);

	/* put scrolled in a framed box so it's more visible */
	hbox1 = mbtk_hbox_new(NULL);
	hbox1->framed = 1;
	mbtk_box_style_set_prop(hbox1, mbtk_kw("bg.color"), mbtk_arg_str("#00ff00")); /* expose empty space around the scrolled window */
	mbtk_box_add_widget(vbox, &hbox1->w, 0);

	scr = mbtk_scrolled_new(NULL);
	mbtk_scrolled_set_minsize(scr, 160, 100);
	mbtk_box_add_widget(hbox1, &scr->w, 0);

	/* control box */
	hbox2 = mbtk_hbox_new(NULL);
	mbtk_box_add_widget(vbox, &hbox2->w, 0);

	btn = mbtk_button_new_with_label("to 0;0");
	mbtk_button_callback_click(btn, scroll_to_0_0, scr);
	mbtk_box_add_widget(hbox2, &btn->w, 0);

	btn = mbtk_button_new_with_label("to 20;10");
	mbtk_button_callback_click(btn, scroll_to_20_10, scr);
	mbtk_box_add_widget(hbox2, &btn->w, 0);

	/* fill in the scrolled */
	scvbox = mbtk_vbox_new(NULL);
	mbtk_scrolled_set_root_widget(scr, &scvbox->w);
	lab = mbtk_label_new(NULL, "This is a real long text so that horizontal scroll can be tested for good");
	mbtk_box_add_widget(scvbox, &lab->w, 0);


	for(stay = 1; stay;) {
		mbtk_event_t ev;
		if (mbtk_iterate(&disp, &ev, 1)) {
			if (ev.type == MBTK_EV_KEY_RELEASE) {
				if (ev.data.key.sym == 'q')
					break;
				if (ev.data.key.sym == '1')
					mbtk_widget_destroy(&scvbox->w);
			}
		}
	}

	mbtk_uninit();
}
